@@ -4,24 +4,32 @@
 #
 # Copyright (C) 2017, James R. Barlow (https://github.com/jbarlow83/)
 
+import struct
+from abc import ABC, abstractmethod
+from decimal import Decimal
 from io import BytesIO
 from itertools import zip_longest
-from abc import ABC, abstractmethod
-import struct
+from pathlib import Path
+from shutil import copyfileobj
+from tempfile import NamedTemporaryFile
+from zlib import decompress
+from zlib import error as ZlibError
 
-from decimal import Decimal
+from .. import Array, Dictionary, Name, Object, PdfError, Stream
 
-from .. import (
-    Object, Array, PdfError, Name, Dictionary, Stream
-)
 
 class DependencyError(Exception):
     pass
 
+
 class UnsupportedImageTypeError(Exception):
     pass
 
 
+class NotExtractableError(Exception):
+    pass
+
+
 def array_str(value):
     if isinstance(value, (list, Array)):
         return [str(item) for item in value]
@@ -37,7 +45,11 @@ def array_str_colorspace(value):
             result = [str(items[n]) for n in range(3)]
             result.append(bytes(items[3]))
             return result
+        if len(items) == 2 and items[0] == '/ICCBased':
+            result = [str(items[0]), items[1]]
+            return result
         return array_str(items)
+
     return array_str(value)
 
 
@@ -112,16 +124,16 @@ class PdfImageBase(ABC):
         if self._colorspaces:
             if self._colorspaces[0] in self.SIMPLE_COLORSPACES:
                 return self._colorspaces[0]
-            if self._colorspaces[0] == '/DeviceCMYK':
+            if self._colorspaces[0] in ('/DeviceCMYK', '/ICCBased'):
                 return self._colorspaces[0]
-            if self._colorspaces[0] == '/Indexed' \
-                    and self._colorspaces[1] in self.SIMPLE_COLORSPACES:
+            if (
+                self._colorspaces[0] == '/Indexed'
+                and self._colorspaces[1] in self.SIMPLE_COLORSPACES
+            ):
                 return self._colorspaces[1]
-            if self._colorspaces[0] == '/ICCBased':
-                icc = self._colorspaces[1]
-                return icc.stream_dict.get('/Alternate', '')
         raise NotImplementedError(
-            "not sure how to get colorspace: " + repr(self._colorspaces))
+            "not sure how to get colorspace: " + repr(self._colorspaces)
+        )
 
     @property
     def bits_per_component(self):
@@ -135,6 +147,11 @@ class PdfImageBase(ABC):
     def is_inline(self):
         pass
 
+    @property
+    @abstractmethod
+    def icc(self):
+        pass
+
     @property
     def indexed(self):
         """``True`` if the image has a defined color palette"""
@@ -147,7 +164,12 @@ class PdfImageBase(ABC):
 
     @property
     def mode(self):
-        """``PIL.Image.mode`` equivalent for this image"""
+        """``PIL.Image.mode`` equivalent for this image, where possible
+
+        If an ICC profile is attached to the image, we still attempt to resolve a Pillow
+        mode.
+        """
+
         m = ''
         if self.indexed:
             m = 'P'
@@ -160,6 +182,18 @@ class PdfImageBase(ABC):
                 m = 'L'
             elif self.colorspace == '/DeviceCMYK':
                 m = 'CMYK'
+            elif self.colorspace == '/ICCBased':
+                try:
+                    icc_profile = self._colorspaces[1]
+                    icc_profile_nchannels = int(icc_profile['/N'])
+                    if icc_profile_nchannels == 1:
+                        m = 'L'
+                    elif icc_profile_nchannels == 3:
+                        m = 'RGB'
+                    elif icc_profile_nchannels == 4:
+                        m = 'CMYK'
+                except (ValueError, TypeError):
+                    pass
         if m == '':
             raise NotImplementedError("Not sure how to handle PDF image of this type")
         return m
@@ -175,7 +209,6 @@ class PdfImageBase(ABC):
         [(/FilterName, {/DecodeParmName: Value, ...}), ...]
 
         The order of /Filter matters as indicates the encoding/decoding sequence.
-
         """
         return list(zip_longest(self.filters, self.decode_parms, fillvalue={}))
 
@@ -183,8 +216,8 @@ class PdfImageBase(ABC):
     def palette(self):
         """Retrieves the color palette for this image
 
-        :returns: (base_colorspace: str, palette: bytes)
-        :rtype: tuple
+        Returns:
+            tuple (base_colorspace: str, palette: bytes)
         """
 
         if not self.indexed:
@@ -209,6 +242,29 @@ class PdfImageBase(ABC):
     def as_pil_image(self):
         pass
 
+    @staticmethod
+    def _unstack_compression(buffer, filters):
+        """Remove stacked compression where it appears.
+
+        Stacked compression means when an image is set to:
+            ``[/FlateDecode /DCTDecode]``
+        for example.
+
+        Only Flate can be stripped off the front currently.
+
+        Args:
+            buffer (pikepdf._qpdf.Buffer): the compressed image data
+            filters (list of str): all files on the data
+        """
+        data = memoryview(buffer)
+        while len(filters) > 1 and filters[0] == '/FlateDecode':
+            try:
+                data = decompress(data)
+            except ZlibError as e:
+                raise UnsupportedImageTypeError() from e
+            filters = filters[1:]
+        return data, filters
+
 
 class PdfImage(PdfImageBase):
     """Support class to provide a consistent API for manipulating PDF images
@@ -237,21 +293,20 @@ class PdfImage(PdfImageBase):
             obj (pikepdf.Object): an Image XObject
 
         """
-        if isinstance(obj, Stream) and \
-                obj.stream_dict.get("/Subtype") != "/Image":
+        if isinstance(obj, Stream) and obj.stream_dict.get("/Subtype") != "/Image":
             raise TypeError("can't construct PdfImage from non-image")
         self.obj = obj
+        self._icc = None
 
     @classmethod
     def _from_pil_image(cls, *, pdf, page, name, image):  # pragma: no cover
         """Insert a PIL image into a PDF (rudimentary)
 
-        :param pdf: the PDF to attach the image to
-        :type pdf: pikepdf.Pdf
-        :param page: the page to attach the image to
-        :param name: the name to set the image
-        :param image: image
-        :type image: PIL.Image.Image
+        Args:
+            pdf (pikepdf.Pdf): the PDF to attach the image to
+            page (pikepdf.Object): the page to attach the image to
+            name (str or pikepdf.Name): the name to set the image
+            image (PIL.Image.Image): the image to insert
         """
 
         data = image.tobytes()
@@ -279,6 +334,26 @@ class PdfImage(PdfImageBase):
         """``False`` for image XObject"""
         return False
 
+    @property
+    def icc(self):
+        """If an ICC profile is attached, return a Pillow object that describe it.
+
+        Most of the information may be found in ``icc.profile``.
+
+        Returns:
+            PIL.ImageCms.ImageCmsProfile
+        """
+        from PIL import ImageCms
+
+        if self.colorspace != '/ICCBased':
+            return None
+        if not self._icc:
+            iccstream = self._colorspaces[1]
+            iccbuffer = iccstream.get_stream_buffer()
+            iccbytesio = BytesIO(iccbuffer)
+            self._icc = ImageCms.ImageCmsProfile(iccbytesio)
+        return self._icc
+
     def _extract_direct(self, *, stream):
         """Attempt to extract the image directly to a usable image file
 
@@ -296,30 +371,38 @@ class PdfImage(PdfImageBase):
             # saved as a standard JPEG. RGB JPEGs without YUV conversion can't
             # be saved as JPEGs, and are probably bugs. Some software in the
             # wild actually produces RGB JPEGs in PDFs (probably a bug).
-            return (self.mode == 'RGB' and
-                    self.filter_decodeparms[0][1].get('/ColorTransform', 1))
+            DEFAULT_CT_RGB = 1
+            ct = self.filter_decodeparms[0][1].get('/ColorTransform', DEFAULT_CT_RGB)
+            return self.mode == 'RGB' and ct == DEFAULT_CT_RGB
 
         def normal_dct_cmyk():
             # Normal DCTDecode CMYKs have /ColorTransform 0 and can be saved.
             # There is a YUVK colorspace but CMYK JPEGs don't generally use it
-            return (self.mode == 'CMYK' and
-                    self.filter_decodeparms[0][1].get('/ColorTransform', 0))
+            DEFAULT_CT_CMYK = 0
+            ct = self.filter_decodeparms[0][1].get('/ColorTransform', DEFAULT_CT_CMYK)
+            return self.mode == 'CMYK' and ct == DEFAULT_CT_CMYK
 
-        if self.filters == ['/CCITTFaxDecode']:
-            data = self.obj.read_raw_bytes()
+        data, filters = self._unstack_compression(
+            self.obj.get_raw_stream_buffer(), self.filters
+        )
+
+        if filters == ['/CCITTFaxDecode']:
+            if self.colorspace == '/ICCBased':
+                raise UnsupportedImageTypeError("Cannot direct-extract CCITT + ICC")
             stream.write(self._generate_ccitt_header(data))
             stream.write(data)
             return '.tif'
-        elif self.filters == ['/DCTDecode'] and (
-                self.mode == 'L' or normal_dct_rgb() or normal_dct_cmyk):
-            buffer = self.obj.get_raw_stream_buffer()
-            stream.write(buffer)
+        elif filters == ['/DCTDecode'] and (
+            self.mode == 'L' or normal_dct_rgb() or normal_dct_cmyk()
+        ):
+            stream.write(data)
             return '.jpg'
 
-        raise UnsupportedImageTypeError()
+        raise NotExtractableError()
 
     def _extract_transcoded(self):
         from PIL import Image
+
         im = None
         if self.mode == 'RGB' and self.bits_per_component == 8:
             # No point in accessing the buffer here, size qpdf decodes to 3-byte
@@ -330,8 +413,7 @@ class PdfImage(PdfImageBase):
             buffer = self.get_stream_buffer()
             stride = 0  # tell Pillow to calculate stride from line width
             ystep = 1  # image is top to bottom in memory
-            im = Image.frombuffer('L', self.size, buffer, "raw", 'L', stride,
-                                  ystep)
+            im = Image.frombuffer('L', self.size, buffer, "raw", 'L', stride, ystep)
             if self.mode == 'P':
                 base_mode, palette = self.palette
                 if base_mode in ('RGB', 'L'):
@@ -347,14 +429,25 @@ class PdfImage(PdfImageBase):
             im = Image.frombytes('1', self.size, data)
 
             base_mode, palette = self.palette
-            if not (palette == b'\x00\x00\x00\xff\xff\xff'
-                    or palette == b'\x00\xff'):
-                raise NotImplementedError(
-                    'monochrome image with nontrivial palette')
+            if base_mode == 'RGB' and palette != b'\x00\x00\x00\xff\xff\xff':
+                im = im.convert('P')
+                im.putpalette(palette, rawmode=base_mode)
+                gp = im.getpalette()
+                gp[765:768] = gp[3:6]  # work around Pillow bug
+                im.putpalette(gp)
+            elif base_mode == 'L' and palette != b'\x00\xff':
+                im = im.convert('P')
+                im.putpalette(palette, rawmode=base_mode)
+                gp = im.getpalette()
+                gp[255] = gp[1]  # work around Pillow bug
+                im.putpalette(gp)
+
+        if self.colorspace == '/ICCBased':
+            im.info['icc_profile'] = self.icc.tobytes()
 
         return im
 
-    def extract_to(self, *, stream):
+    def _extract_to_stream(self, *, stream):
         """Attempt to extract the image directly to a usable image file
 
         If possible, the compressed data is extracted and inserted into
@@ -376,16 +469,69 @@ class PdfImage(PdfImageBase):
 
         try:
             return self._extract_direct(stream=stream)
-        except UnsupportedImageTypeError:
+        except NotExtractableError:
             pass
 
-        im = self._extract_transcoded()
-        if im:
-            im.save(stream, format='png')
-            return '.png'
+        try:
+            im = self._extract_transcoded()
+            if im:
+                im.save(stream, format='png')
+                return '.png'
+        except PdfError as e:
+            if 'getStreamData called on unfilterable stream' in str(e):
+                raise UnsupportedImageTypeError(repr(self)) from e
+            raise
 
         raise UnsupportedImageTypeError(repr(self))
 
+    def extract_to(self, *, stream=None, fileprefix=''):
+        """Attempt to extract the image directly to a usable image file
+
+        If possible, the compressed data is extracted and inserted into
+        a compressed image file format without transcoding the compressed
+        content. If this is not possible, the data will be decompressed
+        and extracted to an appropriate format.
+
+        Because it is not known until attempted what image format will be
+        extracted, users should not assume what format they are getting back.
+        When saving the image to a file, use a temporary filename, and then
+        rename the file to its final name based on the returned file extension.
+
+        Examples:
+
+            >>> im.extract_to(stream=bytes_io)
+            '.png'
+
+            >>> im.extract_to(fileprefix='/tmp/image00')
+            '/tmp/image00.jpg'
+
+        Args:
+            stream: Writable stream to write data to.
+            fileprefix (str or Path): The path to write the extracted image to,
+                without the file extension.
+
+        Returns:
+            If *fileprefix* was provided, then the fileprefix with the
+            appropriate extension. If no *fileprefix*, then an extension
+            indicating the file type.
+
+        Return type:
+            str
+        """
+
+        if bool(stream) == bool(fileprefix):
+            raise ValueError("Cannot set both stream and fileprefix")
+        if stream:
+            return self._extract_to_stream(stream=stream)
+
+        bio = BytesIO()
+        extension = self._extract_to_stream(stream=bio)
+        bio.seek(0)
+        filepath = Path(str(Path(fileprefix)) + extension)
+        with filepath.open('wb') as target:
+            copyfileobj(bio, target)
+        return str(filepath)
+
     def read_bytes(self):
         """Decompress this image and return it as unencoded bytes"""
         return self.obj.read_bytes()
@@ -407,7 +553,7 @@ class PdfImage(PdfImageBase):
             self._extract_direct(stream=bio)
             bio.seek(0)
             return Image.open(bio)
-        except UnsupportedImageTypeError:
+        except NotExtractableError:
             pass
 
         im = self._extract_transcoded()
@@ -424,15 +570,28 @@ class PdfImage(PdfImageBase):
         if not self.decode_parms:
             raise ValueError("/CCITTFaxDecode without /DecodeParms")
 
-        if self.decode_parms[0].get("/K", 1) < 0:
+        if self.decode_parms[0].get("/EncodedByteAlign", False):
+            raise UnsupportedImageTypeError(
+                "/CCITTFaxDecode with /EncodedByteAlign true"
+            )
+
+        k = self.decode_parms[0].get("/K", 0)
+        if k < 0:
             ccitt_group = 4  # Pure two-dimensional encoding (Group 4)
+        elif k > 0:
+            ccitt_group = 3  # Group 3 2-D
         else:
-            ccitt_group = 3
+            ccitt_group = 2  # CCITT 1-D
         black_is_one = self.decode_parms[0].get("/BlackIs1", False)
-        white_is_zero = 1 if black_is_one else 0
+        # TIFF spec says:
+        # use 0 for white_is_zero (=> black is 1)
+        # use 1 for black_is_zero (=> white is 1)
+        # In practice, do the opposite of what the TIFF spec says.
+        photometry = 1 if black_is_one else 0
 
         img_size = len(data)
         tiff_header_struct = '<' + '2s' + 'H' + 'L' + 'H' + 'HHLL' * 8 + 'L'
+        # fmt: off
         tiff_header = struct.pack(
             tiff_header_struct,
             b'II',  # Byte order indication: Little endian
@@ -443,12 +602,13 @@ class PdfImage(PdfImageBase):
             257, 4, 1, self.height,  # ImageLength, LONG, 1, length
             258, 3, 1, 1,  # BitsPerSample, SHORT, 1, 1
             259, 3, 1, ccitt_group,  # Compression, SHORT, 1, 4 = CCITT Group 4 fax encoding
-            262, 3, 1, int(white_is_zero),  # Thresholding, SHORT, 1, 0 = WhiteIsZero
+            262, 3, 1, int(photometry),  # Thresholding, SHORT, 1, 0 = WhiteIsZero
             273, 4, 1, struct.calcsize(tiff_header_struct),  # StripOffsets, LONG, 1, length of header
             278, 4, 1, self.height,
             279, 4, 1, img_size,  # StripByteCounts, LONG, 1, size of image
             0  # last IFD
         )
+        # fmt: on
         return tiff_header
 
     def show(self):
@@ -457,7 +617,8 @@ class PdfImage(PdfImageBase):
 
     def __repr__(self):
         return '<pikepdf.PdfImage image mode={} size={}x{} at {}>'.format(
-            self.mode, self.width, self.height, hex(id(self)))
+            self.mode, self.width, self.height, hex(id(self))
+        )
 
     def _repr_png_(self):
         """Display hook for IPython/Jupyter"""
@@ -468,14 +629,17 @@ class PdfImage(PdfImageBase):
 
 
 class PdfJpxImage(PdfImage):
-
     def __init__(self, obj):
         super().__init__(obj)
         self.pil = self.as_pil_image()
 
     def _extract_direct(self, *, stream):
-        buffer = self.obj.get_raw_stream_buffer()
-        stream.write(buffer)
+        data, filters = self._unstack_compression(
+            self.obj.get_raw_stream_buffer(), self.filters
+        )
+        if filters != ['/JPXDecode']:
+            raise UnsupportedImageTypeError(self.filters)
+        stream.write(data)
         return '.jp2'
 
     @property
@@ -508,7 +672,8 @@ class PdfJpxImage(PdfImage):
 
     def __repr__(self):
         return '<pikepdf.PdfJpxImage JPEG2000 image mode={} size={}x{} at {}>'.format(
-            self.mode, self.width, self.height, hex(id(self)))
+            self.mode, self.width, self.height, hex(id(self))
+        )
 
 
 class PdfInlineImage(PdfImageBase):
@@ -532,7 +697,7 @@ class PdfInlineImage(PdfImageBase):
         b'/LZW': b'/LZWDecode',
         b'/RL': b'/RunLengthDecode',
         b'/CCF': b'/CCITTFaxDecode',
-        b'/DCT': b'/DCTDecode'
+        b'/DCT': b'/DCTDecode',
     }
 
     def __init__(self, *, image_data, image_object: tuple):
@@ -554,8 +719,7 @@ class PdfInlineImage(PdfImageBase):
         try:
             reparsed_obj = Object.parse(b'<< ' + reparse + b' >>')
         except PdfError as e:
-            raise PdfError(
-                "parsing inline " + reparse.decode('unicode_escape')) from e
+            raise PdfError("parsing inline " + reparse.decode('unicode_escape')) from e
         self.obj = reparsed_obj
         self.pil = None
 
@@ -575,7 +739,6 @@ class PdfInlineImage(PdfImageBase):
         else:
             raise NotImplementedError(repr(obj))
 
-
     def _metadata(self, name, type_, default):
         return metadata_from_obj(self.obj, name, type_, default)
 
@@ -597,6 +760,10 @@ class PdfInlineImage(PdfImageBase):
     def is_inline(self):
         return True
 
+    @property
+    def icc(self):
+        raise ValueError("Inline images may not have ICC profiles")
+
     def __repr__(self):
         mode = '?'
         try:
@@ -604,23 +771,24 @@ class PdfInlineImage(PdfImageBase):
         except Exception:
             pass
         return '<pikepdf.PdfInlineImage image mode={} size={}x{} at {}>'.format(
-            mode, self.width, self.height, hex(id(self)))
+            mode, self.width, self.height, hex(id(self))
+        )
 
     def as_pil_image(self):
-        from PIL import Image
-
         if self.pil:
             return self.pil
 
         raise NotImplementedError('not yet')
 
-    def extract_to(self, *, stream):  # pylint: disable=unused-argument
+    def extract_to(
+        self, *, stream=None, fileprefix=''
+    ):  # pylint: disable=unused-argument
         raise UnsupportedImageTypeError("inline images don't support extract")
 
     def read_bytes(self):
         raise NotImplementedError("qpdf returns compressed")
-        #return self._data._inline_image_bytes()
+        # return self._data._inline_image_bytes()
 
     def get_stream_buffer(self):
         raise NotImplementedError("qpdf returns compressed")
-        #return memoryview(self._data.inline_image_bytes())
+        # return memoryview(self._data.inline_image_bytes())
