@@ -4,24 +4,21 @@
 #
 # Copyright (C) 2018, James R. Barlow (https://github.com/jbarlow83/)
 
+import logging
+import re
+import sys
 from collections import namedtuple
 from collections.abc import MutableMapping
 from datetime import datetime
 from functools import wraps
 from io import BytesIO
-from pkg_resources import (
-    get_distribution as _get_distribution,
-    DistributionNotFound
-)
 from warnings import warn
-import re
-import sys
 
 from lxml import etree
-from lxml.etree import QName, XMLSyntaxError
-from defusedxml.lxml import parse
+from lxml.etree import QName, XMLParser, XMLSyntaxError, parse
 
-from .. import Stream, Name, String, PdfError
+from .. import Name, PdfError, Stream, String
+from .. import __version__ as pikepdf_version
 
 XMP_NS_DC = "http://purl.org/dc/elements/1.1/"
 XMP_NS_PDF = "http://ns.adobe.com/pdf/1.3/"
@@ -68,6 +65,14 @@ XPACKET_END = b"""\n<?xpacket end="w"?>\n"""
 
 XmpContainer = namedtuple('XmpContainer', ['rdf_type', 'py_type', 'insert_fn'])
 
+log = logging.getLogger(__name__)
+
+
+class NeverRaise(Exception):
+    """An exception that is never raised"""
+
+    pass  # pylint: disable=unnecessary-pass
+
 
 class AltList(list):
     pass
@@ -79,12 +84,14 @@ XMP_CONTAINERS = [
     XmpContainer('Seq', list, list.append),
 ]
 
-LANG_ALTS = frozenset([
-    str(QName(XMP_NS_DC, 'title')),
-    str(QName(XMP_NS_DC, 'description')),
-    str(QName(XMP_NS_DC, 'rights')),
-    str(QName(XMP_NS_XMP_RIGHTS, 'UsageTerms')),
-])
+LANG_ALTS = frozenset(
+    [
+        str(QName(XMP_NS_DC, 'title')),
+        str(QName(XMP_NS_DC, 'description')),
+        str(QName(XMP_NS_DC, 'rights')),
+        str(QName(XMP_NS_XMP_RIGHTS, 'UsageTerms')),
+    ]
+)
 
 # These are the illegal characters in XML 1.0. (XML 1.1 is a bit more permissive,
 # but we'll be strict to ensure wider compatibility.)
@@ -93,14 +100,9 @@ re_xml_illegal_chars = re.compile(
 )
 re_xml_illegal_bytes = re.compile(
     br"[^\x09\x0A\x0D\x20-\xFF]|&#0;"
+    # br"&#(?:[0-9]|0[0-9]|1[0-9]|2[0-9]|3[0-1]|x[0-9A-Fa-f]|x0[0-9A-Fa-f]|x1[0-9A-Fa-f]);"
 )
 
-# Repeat this to avoid circular from top package's pikepdf.__version__
-try:
-    pikepdf_version = _get_distribution(__name__).version
-except DistributionNotFound:
-    pikepdf_version = ""
-
 
 def encode_pdf_date(d: datetime) -> str:
     """Encode Python datetime object as PDF date string
@@ -121,8 +123,11 @@ def encode_pdf_date(d: datetime) -> str:
     the local time.
     """
 
-    pdfmark_date_fmt = r'%Y%m%d%H%M%S'
-    s = d.strftime(pdfmark_date_fmt)
+    # The formatting of %Y is not consistent as described in
+    # https://bugs.python.org/issue13305 and underspecification in libc.
+    # So explicitly format the year with leading zeros
+    s = "{:04d}".format(d.year)
+    s += d.strftime(r'%m%d%H%M%S')
     tz = d.strftime('%z')
     if tz:
         sign, tz_hours, tz_mins = tz[0], tz[1:3], tz[3:5]
@@ -168,6 +173,7 @@ class AuthorConverter:
 
 
 if sys.version_info < (3, 7):
+
     def fromisoformat(datestr):
         # strptime %z can't parse a timezone with punctuation
         if re.search(r'[+-]\d{2}[-:]\d{2}$', datestr):
@@ -176,9 +182,12 @@ if sys.version_info < (3, 7):
             return datetime.strptime(datestr, "%Y-%m-%dT%H:%M:%S%z")
         except ValueError:
             return datetime.strptime(datestr, "%Y-%m-%dT%H:%M:%S")
+
+
 else:
     fromisoformat = datetime.fromisoformat
 
+
 class DateConverter:
     @staticmethod
     def xmp_from_docinfo(docinfo_val):
@@ -200,6 +209,7 @@ def ensure_loaded(fn):
         if not self._xmp:
             self._load()
         return fn(self, *args, **kwargs)
+
     return wrapper
 
 
@@ -225,10 +235,10 @@ class PdfMetadata(MutableMapping):
 
     To update metadata, use a with block.
 
-    .. code-block:: python
+    Example:
 
-        with pdf.open_metadata() as records:
-            records['dc:title'] = 'New Title'
+        >>> with pdf.open_metadata() as records:
+                records['dc:title'] = 'New Title'
 
     See Also:
         :meth:`pikepdf.Pdf.open_metadata`
@@ -248,12 +258,15 @@ class PdfMetadata(MutableMapping):
     NS = {prefix: uri for uri, prefix in DEFAULT_NAMESPACES}
     REVERSE_NS = {uri: prefix for uri, prefix in DEFAULT_NAMESPACES}
 
-    def __init__(self, pdf, pikepdf_mark=True, sync_docinfo=True):
+    def __init__(
+        self, pdf, pikepdf_mark=True, sync_docinfo=True, overwrite_invalid_xml=True
+    ):
         self._pdf = pdf
         self._xmp = None
         self.mark = pikepdf_mark
         self.sync_docinfo = sync_docinfo
         self._updating = False
+        self.overwrite_invalid_xml = overwrite_invalid_xml
 
     def load_from_docinfo(self, docinfo, delete_missing=False, raise_failure=False):
         """Populate the XMP metadata object with DocumentInfo
@@ -286,11 +299,26 @@ class PdfMetadata(MutableMapping):
                     continue
                 self[qname] = val
             except (ValueError, AttributeError) as e:
-                msg = "The metadata field {} could not be copied to XMP".format(docinfo_name)
+                msg = "The metadata field {} could not be copied to XMP".format(
+                    docinfo_name
+                )
                 if raise_failure:
                     raise ValueError(msg) from e
                 else:
                     warn(msg)
+        valid_docinfo_names = set(
+            str(docinfo_name) for _, _, docinfo_name, _ in self.DOCINFO_MAPPING
+        )
+        extra_docinfo_names = set(str(k) for k in docinfo.keys()) - valid_docinfo_names
+        for extra in extra_docinfo_names:
+            msg = (
+                "The metadata field {} with value '{}' has no XMP equivalent, "
+                "so it was discarded"
+            ).format(extra, docinfo.get(extra))
+            if raise_failure:
+                raise ValueError(msg)
+            else:
+                warn(msg)
 
     def _load(self):
         try:
@@ -300,17 +328,54 @@ class PdfMetadata(MutableMapping):
         self._load_from(data)
 
     def _load_from(self, data):
-        try:
-            self._xmp = parse(BytesIO(data))
-        except XMLSyntaxError:
-            data = re_xml_illegal_bytes.sub(b'', data)
+        if data.strip() == b'':
+            data = XMP_EMPTY  # on some platforms lxml chokes on empty documents
+
+        def basic_parser(xml):
+            return parse(BytesIO(xml))
+
+        def strip_illegal_bytes_parser(xml):
+            return parse(BytesIO(re_xml_illegal_bytes.sub(b'', xml)))
+
+        def recovery_parser(xml):
+            parser = XMLParser(recover=True)
+            return parse(BytesIO(xml), parser)
+
+        def replace_with_empty_xmp(xml=None):
+            log.warning("Error occurred parsing XMP, replacing with empty XMP.")
+            return basic_parser(XMP_EMPTY)
+
+        if self.overwrite_invalid_xml:
+            parsers = [
+                basic_parser,
+                strip_illegal_bytes_parser,
+                recovery_parser,
+                replace_with_empty_xmp,
+            ]
+        else:
+            parsers = [basic_parser]
+
+        for parser in parsers:
             try:
-                self._xmp = parse(BytesIO(data))
-            except XMLSyntaxError as e:
-                raise PdfError() from e
-        pis = self._xmp.xpath('/processing-instruction()')
-        for pi in pis:
-            etree.strip_tags(self._xmp, pi.tag)
+                self._xmp = parser(data)
+            except (XMLSyntaxError if self.overwrite_invalid_xml else NeverRaise) as e:
+                if str(e).startswith("Start tag expected, '<' not found") or str(
+                    e
+                ).startswith("Document is empty"):
+                    self._xmp = replace_with_empty_xmp()
+                    break
+            else:
+                break
+
+        try:
+            pis = self._xmp.xpath('/processing-instruction()')
+            for pi in pis:
+                etree.strip_tags(self._xmp, pi.tag)
+            self._get_rdf_root()
+        except (Exception if self.overwrite_invalid_xml else NeverRaise) as e:
+            log.warning("Error occurred parsing XMP", exc_info=e)
+            self._xmp = replace_with_empty_xmp()
+        return
 
     @ensure_loaded
     def __enter__(self):
@@ -344,7 +409,11 @@ class PdfMetadata(MutableMapping):
                 try:
                     value = converter.docinfo_from_xmp(value)
                 except ValueError:
-                    warn("The DocumentInfo field {} could not be updated from XMP".format(docinfo_name))
+                    warn(
+                        "The DocumentInfo field {} could not be updated from XMP".format(
+                            docinfo_name
+                        )
+                    )
                     value = None
             if value is None:
                 if docinfo_name in self._pdf.docinfo:
@@ -434,6 +503,14 @@ class PdfMetadata(MutableMapping):
             return result
         return ''
 
+    def _get_rdf_root(self):
+        rdf = self._xmp.find('.//rdf:RDF', self.NS)
+        if rdf is None:
+            rdf = self._xmp.getroot()
+            if not rdf.tag == '{http://www.w3.org/1999/02/22-rdf-syntax-ns#}RDF':
+                raise ValueError("Metadata seems to be XML but not XMP")
+        return rdf
+
     def _get_elements(self, name=''):
         """Get elements from XMP
 
@@ -458,7 +535,7 @@ class PdfMetadata(MutableMapping):
 
         """
         qname = self._qname(name)
-        rdf = self._xmp.find('.//rdf:RDF', self.NS)
+        rdf = self._get_rdf_root()
         for rdfdesc in rdf.findall('rdf:Description[@rdf:about=""]', self.NS):
             if qname and qname in rdfdesc.keys():
                 yield (rdfdesc, qname, rdfdesc.get(qname), rdf)
@@ -546,24 +623,24 @@ class PdfMetadata(MutableMapping):
                 raise TypeError(val)
         except StopIteration:
             # Insert a new node
-            rdf = self._xmp.find('.//rdf:RDF', self.NS)
+            rdf = self._get_rdf_root()
             if str(self._qname(key)) in LANG_ALTS:
                 val = AltList([clean(val)])
             if isinstance(val, (list, set)):
                 rdfdesc = etree.SubElement(
-                    rdf, QName(XMP_NS_RDF, 'Description'),
-                    attrib={
-                        QName(XMP_NS_RDF, 'about'): '',
-                    },
+                    rdf,
+                    QName(XMP_NS_RDF, 'Description'),
+                    attrib={QName(XMP_NS_RDF, 'about'): ''},
                 )
                 node = etree.SubElement(rdfdesc, self._qname(key))
                 add_array(node, val)
             elif isinstance(val, str):
                 rdfdesc = etree.SubElement(
-                    rdf, QName(XMP_NS_RDF, 'Description'),
+                    rdf,
+                    QName(XMP_NS_RDF, 'Description'),
                     attrib={
                         QName(XMP_NS_RDF, 'about'): '',
-                        self._qname(key): clean(val)
+                        self._qname(key): clean(val),
                     },
                 )
             else:
@@ -577,7 +654,11 @@ class PdfMetadata(MutableMapping):
             node, attrib, _oldval, parent = next(self._get_elements(key))
             if attrib:  # Inline
                 del node.attrib[attrib]
-                if len(node.attrib) == 1 and len(node) == 0 and QName(XMP_NS_RDF, 'about') in node.attrib:
+                if (
+                    len(node.attrib) == 1
+                    and len(node) == 0
+                    and QName(XMP_NS_RDF, 'about') in node.attrib
+                ):
                     # The only thing left on this node is rdf:about="", so remove it
                     parent.remove(node)
             else:
