@@ -1,12 +1,8 @@
 import os
 import sys
-import platform
-
-pytest_plugins = ['helpers_namespace']
+from pathlib import Path
 
 import pytest
-from pathlib import Path
-from subprocess import Popen, PIPE
 
 
 if sys.version_info < (3, 4):
@@ -24,5 +20,10 @@ def resources():
 
 
 @pytest.fixture(scope="function")
-def outdir(tmpdir):
-    return Path(str(tmpdir))
+def outdir(tmp_path):
+    return tmp_path
+
+
+@pytest.fixture(scope="function")
+def outpdf(tmp_path):
+    return tmp_path / 'out.pdf'
