@@ -1,12 +1,16 @@
-import re
-from datetime import datetime, timedelta, timezone
 import os
+import platform
+import re
+import sys
+import xml.etree.ElementTree as ET
+from datetime import datetime, timedelta, timezone
 from pathlib import Path
 
-import xml.etree.ElementTree as ET
 import pytest
-from hypothesis import given, example
+from hypothesis import assume, example, given
+from hypothesis import strategies as st
 from hypothesis.strategies import integers
+from lxml.etree import XMLSyntaxError
 
 import pikepdf
 from pikepdf import Dictionary, Name, PasswordError, Pdf, Stream
@@ -15,6 +19,7 @@ from pikepdf.models.metadata import (
     XMP_NS_PDF,
     XMP_NS_XMP,
     DateConverter,
+    PdfMetadata,
     decode_pdf_date,
     encode_pdf_date,
 )
@@ -360,7 +365,7 @@ def test_wrong_xml(enron1):
         <test><xml>This is valid xml but not valid XMP</xml></test>
     """.strip(),
     )
-    meta = enron1.open_metadata()
+    meta = enron1.open_metadata(strict=True)
     with pytest.raises(ValueError, match='not XMP'):
         with meta:
             pass
@@ -396,18 +401,55 @@ def test_no_x_xmpmeta(trivial):
     assert xmp['pdfaid:part'] == '2'
 
 
-def test_empty_xmpmeta(trivial):
-    trivial.Root.Metadata = Stream(
-        trivial,
-        b"""<?xpacket begin="" id=""?>
-        <x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="">
-        </x:xmpmeta>
-        <?xpacket end=""?>
-        """,
-    )
-    with trivial.open_metadata() as xmp:
+@pytest.mark.parametrize(
+    'xml',
+    [
+        (b"      \n   "),
+        (b" <"),
+        (
+            b'<?xpacket begin="\xef\xbb\xbf" id="W5M0MpCehiHzreSzNTczkc9d"?>\n'
+            b'<?xpacket end=""?>\n'
+        ),
+        (
+            b'<?xpacket begin="" id=""?>\n'
+            b'<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="">\n'
+            b'</x:xmpmeta>\n'
+            b'<?xpacket end=""?>\n'
+        ),
+    ],
+)
+def test_degenerate_xml_recoverable(trivial, xml):
+    trivial.Root.Metadata = trivial.make_stream(xml)
+    with trivial.open_metadata(strict=False) as xmp:
+        xmp['pdfaid:part'] = '2'
+    assert xmp['pdfaid:part'] == '2'
+
+    with trivial.open_metadata(strict=True) as xmp:
+        xmp['pdfaid:part'] = '5'
+
+
+@given(st.integers(min_value=1, max_value=1350))
+@example(548)
+@example(1154)
+@example(1155)
+@example(1195)
+@example(1303)
+@pytest.mark.filterwarnings('ignore:The DocumentInfo field')
+def test_truncated_xml(resources, idx):
+    sandwich = Pdf.open(resources / 'sandwich.pdf')
+    data = sandwich.Root.Metadata.read_bytes()
+    assume(idx < len(data))
+
+    sandwich.Root.Metadata = sandwich.make_stream(data[0:idx])
+    try:
+        with sandwich.open_metadata(strict=True) as xmp:
+            xmp['pdfaid:part'] = '5'
+    except (XMLSyntaxError, AssertionError):
         pass
 
+    with sandwich.open_metadata(strict=False) as xmp:
+        xmp['pdfaid:part'] = '7'
+
 
 @needs_libxmp
 def test_pdf_version_update(graph, outdir):
@@ -455,3 +497,34 @@ def test_extension_level(trivial, outpdf):
 
     with pytest.raises(TypeError):
         trivial.save(outpdf, force_version=('1.7', 'invalid extension level'))
+
+
+@given(
+    st.dictionaries(
+        keys=st.sampled_from(
+            [
+                "/Author",
+                "/Subject",
+                "/Title",
+                "/Keywords",
+                "/Producer",
+                "/CreationDate",
+                "/Creator",
+                "/ModDate",
+                "/Dummy",
+            ]
+        ),
+        values=st.binary(),
+    )
+)
+def test_random_docinfo(docinfo):
+    p = pikepdf.new()
+    with p.open_metadata() as m:
+        pdf_docinfo = pikepdf.Dictionary(docinfo)
+
+        try:
+            m.load_from_docinfo(pdf_docinfo, raise_failure=True)
+        except ValueError as e:
+            assert 'could not be copied to XMP' in str(e) or '/Dummy' in str(e)
+        else:
+            ET.fromstring(str(m))  # ensure we can parse it
