@@ -5,7 +5,7 @@ Testing focused on pikepdf.Pdf
 import os
 import shutil
 import sys
-from io import StringIO
+from io import StringIO, BytesIO
 from pathlib import Path
 from unittest.mock import Mock, patch
 
@@ -31,7 +31,18 @@ def test_new(outdir):
 
 def test_non_filename():
     with pytest.raises(TypeError):
-        Pdf.open(42)
+        Pdf.open(42.0)
+
+
+def test_file_descriptor(resources):
+    with (resources / 'pal-1bit-trivial.pdf').open('rb') as f:
+        with pytest.raises(TypeError):
+            pdf = Pdf.open(f.fileno())
+
+
+def test_save_to_file_descriptor_fails(trivial):
+    with pytest.raises(TypeError):
+        trivial.save(2)
 
 
 def test_not_existing_file():
@@ -119,6 +130,22 @@ class TestStreams:
             saved_file_contents = saved_file.read()
         assert saved_file_contents == bio.read()
 
+    @pytest.mark.skipif(sys.version_info < (3, 6), reason="pathlib on 3.5")
+    def test_read_not_readable_file(self, outdir):
+        writable = (Path(outdir) / 'writeme.pdf').open('wb')
+        with pytest.raises(ValueError, match=r'not readable'):
+            Pdf.open(writable)
+
+    def test_open_not_seekable_stream(self, resources):
+        class UnseekableBytesIO(BytesIO):
+            def seekable(self):
+                return False
+
+        testio = UnseekableBytesIO((resources / 'pal-1bit-trivial.pdf').read_bytes())
+
+        with pytest.raises(ValueError, match=r'not seekable'):
+            Pdf.open(testio)
+
 
 class TestMemory:
     def test_memory(self, resources):
@@ -162,19 +189,6 @@ def test_unicode_filename(resources, outdir):
     assert target2.exists()
 
 
-@pytest.mark.skipif(os.name == 'nt', reason='os.dup hackery not supported')
-def test_fileno_fails(resources):
-    with patch('os.dup') as dup:
-        dup.side_effect = OSError('assume dup fails')
-        with pytest.raises(OSError):
-            Pdf.open(resources / 'pal-1bit-trivial.pdf')
-
-    with patch('os.dup') as dup:
-        dup.return_value = -1
-        with pytest.raises(RuntimeError):
-            Pdf.open(resources / 'pal-1bit-trivial.pdf')
-
-
 def test_min_and_force_version(trivial, outdir):
     pdf = trivial
     pdf.save(outdir / '1.7.pdf', min_version='1.7')
@@ -238,3 +252,14 @@ def test_with_block_abuse(resources):
         im0 = pdf.pages[0].Resources.XObject['/Im0']
     with pytest.raises(PdfError):
         im0.read_bytes()
+
+
+def test_check(resources):
+    with pikepdf.open(resources / 'content-stream-errors.pdf') as pdf:
+        problems = pdf.check()
+        assert all(isinstance(prob, str) for prob in problems)
+        assert 'parse error while reading' in problems[0]
+
+
+def test_repr(trivial):
+    assert repr(trivial).startswith('<')
